#include <Wire.h>
#include <ADXL345.h> 
#include "remotexy.h"

ADXL345 adxl; //variable adxl is an instance of the ADXL345 library

// пины моторов
// левый мотор
int IN1 = 6;
int IN2 = 7;
int ENA = 3;
//правый мотор
int IN3 = 9;
int IN4 = 10;
int ENB = 11;

//boolean StopMotors = false; // переменная учета остановки
boolean balancer = false; // переменная срабатывания балансировки
boolean balancerForward = false; // переменная срабатывания балансировки при движении вперед
boolean balancerBackward = false; // переменная срабатывания балансировки при движении назад
boolean rframeMove = false; // переменная движения (набора скорости)
int x,y,z; // координаты акселерометра


int balanceXMax = 100; // предельный угол наклона
int defmoveSpeed = 40; // начальная скорость по умолчанию
int moveSpeed = 0; // скорость движения
int moveSpeedMax = 255; // максимальная скорость движения
int moveSpeedDelay = 10; // задержка до изменения скорости при движении

unsigned long previousSpeedMillis; // для таймера набора скорости

int LeftMotorSpeed = 255; // скорость левого мотора
int RightMotorSpeed = 255; // скорость правого мотора

void setup() 
{
  //Serial.begin(9600);
  RemoteXY_Init (); // RemoteXY

  // инициализация пинов моторшилда
  pinMode (ENA, OUTPUT);
  pinMode (IN1, OUTPUT);
  pinMode (IN2, OUTPUT);
  pinMode (ENB, OUTPUT);
  pinMode (IN4, OUTPUT);
  pinMode (IN3, OUTPUT);

  adxl.powerOn(); // инициализация акселерометра

}

void loop() 
{ 
  unsigned long currentMillis = millis(); // записываем текущее время
  RemoteXY_Handler (); // RemoteXY

  //Boring accelerometer stuff     
  adxl.readAccel(&x, &y, &z); //read the accelerometer values and store them in variables  x,y,z
  /*
  Serial.print("x: ");
   Serial.print(x);
   Serial.print(" ");
   Serial.print("y: ");
   Serial.print(y);
   Serial.print(" ");
   Serial.print("z: ");
   Serial.print(z);
   Serial.print(" ");
   Serial.print("B: ");
   Serial.print(balancer);
   Serial.print(" ");
   Serial.print("S: ");
   Serial.print(moveSpeed);
   Serial.println(" ");
   */

  // если робот двигается (управляется) и превышено время задержки набора скорости и скорость меньше максимальной и не сработала балансировка
  if(rframeMove && currentMillis - previousSpeedMillis > moveSpeedDelay && moveSpeed < moveSpeedMax && !balancer){
    previousSpeedMillis = currentMillis; // записываем текущее время
    // если скорость меньше чем по умолчанию, задаем скорость по умолчанию
    if (moveSpeed < defmoveSpeed){
      moveSpeed = defmoveSpeed;
    }
    // если скорость >= 80 и меньше 250, прирост скорости идет по 5 единиц
    if(moveSpeed >= 80 && moveSpeed < 250){
      moveSpeed += 5; // увеличение скорости по 5 единиц
    }
    else {
      moveSpeed ++; // увеличение скорости по 1 единице
    }
  }


  // Движение вперед
  // если бегунок джойстика перемещается вверх и не срабатывает балансировка при движении вперед
  if (RemoteXY.joystick_1_y > 0 && !balancerForward) {
    rframeMove = true; // разрешаем набор скорости

    // поворот направо
    // если бегунок джойстика сдвинут вправо по оси X
    if (RemoteXY.joystick_1_x > 0){
      RightMotorSpeed = (100 - RemoteXY.joystick_1_x)*2.55; // скорость правого мотора снижается в зависимости от положения джойстика по оси X
    } 
    else {
      RightMotorSpeed = moveSpeed; // скорость правого мотора равен текущей скорости
    }

    // поворот налево
    // если бегунок джойстика сдвинут влево по оси X
    if (RemoteXY.joystick_1_x < 0){
      LeftMotorSpeed = (100 - (-RemoteXY.joystick_1_x))*2.55; // скорость левого мотора снижается в зависимости от положения джойстика по оси X
    } 
    else {
      LeftMotorSpeed = moveSpeed; // скорость левого мотора равен текущей скорости
    }
    // едет вперед
    digitalWrite(IN4, HIGH); 
    digitalWrite(IN3, LOW); 
    analogWrite(ENB, RightMotorSpeed); // задаем скорость правого мотора
    digitalWrite(IN2, HIGH); 
    digitalWrite(IN1, LOW); 
    analogWrite(ENA, LeftMotorSpeed); // задаем скорость левого мотора
  }

  // Движение назад
  // повороты не сделаны
  if (RemoteXY.joystick_1_y < 0) {
    rframeMove = true; // разрешаем набор скорости
    digitalWrite(IN2, LOW); 
    digitalWrite(IN1, HIGH); 
    analogWrite(ENA, moveSpeed);
    digitalWrite(IN4, LOW); 
    digitalWrite(IN3, HIGH); 
    analogWrite(ENB, moveSpeed);
  }

  // торможение
  // если джойстик отпущен (занимает центральное положение автоматически)
  if (RemoteXY.joystick_1_y == 0){
    moveSpeed = defmoveSpeed;
    analogWrite(ENB, 0); // остановка правого мотора
    analogWrite(ENA, 0); // остановка левого мотора

    // остановка не управляемого движения робота при помощи балансировки
    // если роботом не управляют и он катится по инерции, балансировка гасит его движение
    // выбрана минимальная скорость при которой робот почти не может сдвинутся с места самостоятельно
    // это позволяет гасить движение без прокручивания внутренней части (торможение противоходом)

    // если сработала балансировка при движении вперед, моторы крутят в обратную сторону (крутят назад)
    if(balancerForward){
      digitalWrite(IN4, LOW); 
      digitalWrite(IN3, HIGH); 
      analogWrite(ENB, 50);
      digitalWrite(IN2, LOW); 
      digitalWrite(IN1, HIGH);  
      analogWrite(ENA, 50);
    }
    // если сработала балансировка при движении назад, моторы крутят в обратную сторону (крутят вперед)
    if(balancerBackward){
      digitalWrite(IN2, HIGH); 
      digitalWrite(IN1, LOW);  
      analogWrite(ENA, 50);
      digitalWrite(IN4, HIGH); 
      digitalWrite(IN3, LOW); 
      analogWrite(ENB, 50);
    }
  }

  // балансировка во время движения вперед
  // если при движении вперед внутренняя часть начинает прокручиваться (отклоняться более заданного угла balanceXMax), то скорость снижается на 10 единиц
  // если внутренняя часть возвращается в рабочие границы отклонения, робот продолжает движение и скорость плавно возрастает
  // это обеспечивает плавный старт и мягкую балансировку во время движения
  if(x < 0 && x < -balanceXMax && !balancerForward){
    balancerForward = true; // включается балансировка при движении вперед
    rframeMove = false; // запрещаем набор скорости
    balancer = true; // указываем что работает балансировка
    moveSpeed -= 10; // снижаем скорость
    analogWrite(ENA, moveSpeed); // задаем скорость правого мотора
    analogWrite(ENB, moveSpeed); // задаем скорость левого мотора
  }
  // балансировка во время движения назад (не доделаны)
  if(x > 0 && x > balanceXMax && !balancerBackward){
    balancerBackward = true;
    rframeMove = false;
    balancer = true;
    moveSpeed -= 10;
    analogWrite(ENA, defmoveSpeed);
    analogWrite(ENB, defmoveSpeed);
  }

  // если внутренняя часть находится в пределах угла отклонения -balanceXMax и  balanceXMax, отключаем балансировку
  if(x > -balanceXMax && x < balanceXMax){
    balancerBackward = false;
    balancerForward = false;
    balancer = false;
  }


  // вывод на экран приложения, использовал во время настройки
  itoa (RemoteXY.joystick_1_y, RemoteXY.text_1, 10); // показатели джойстика по оси Y
  itoa (RemoteXY.joystick_1_x, RemoteXY.text_2, 10); // показатели джойстика по оси X

} // конец loop

